/*
 * Copyright (c) 2025. Sergey S. Chernov - All Rights Reserved
 *
 *  You may use, distribute and modify this code under the
 * terms of the private license, which you must obtain from the author
 *
 * To obtain the license, contact the author: https://t.me/real_sergeych or email to
 * real dot sergeych at gmail.
 */

@file:Suppress("unused")

package net.sergeych.tools

import kotlinx.coroutines.cancel
import kotlinx.coroutines.coroutineScope
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.launch

/**
 * suspend until the flow produces the value to which the
 * predicate returns true
 */
suspend fun <T>Flow<T>.waitFor(predicate: (T)->Boolean) {
    coroutineScope {
        launch {
            collect {
                if( predicate(it) ) cancel()
            }
        }
    }
}